@@ -17,20 +17,26 @@ binmode *STDOUT, ":encoding(${encoding})";
 
 my $history_file_name = 'suddenly.hist';
 
+my @dict = (
+    'hello there',
+    'mattn',
+    'tokuhirom'
+);
+
 my $c = Caroline->new(
     completion_callback => sub {
-        my ($line) = @_;
-        if ($line eq 'h') {
-            return (
-                'hello',
-                'hello there'
-            );
-        } elsif ($line eq 'm') {
-            return (
-                '突然のmattn'
-            );
+        my ($line, $pos) = @_;
+
+        # look last word
+        my $part = pop @{[split(/\s+/, substr($line, 0, $pos))]};
+        my $found = '';
+        foreach my $item (@dict) {
+            if ($item =~ /^\Q$part\E/) {
+                $found = $item;
+                last;
+            }
         }
-        return;
+        return substr($line, 0, $pos - length $part) . $found . substr($line, $pos);
     },
     history_max_len => 30,
 );
@@ -40,7 +46,7 @@ if (-f $history_file_name) {
 }
 while (defined(my $line = $c->readline('hello> '))) {
     if ($line =~ /\S/) {
-        print sudden_death($line), "\n";
+        print sudden_death("突然の$line"), "\n";
         $c->history_add($line);
     }
 }
